/*
extremergb.c - finds extreme values of R, G, B channels
Most of this code is taken dorectly from resample.c
please report bugs and/or enhancements to:
<cej@gli.cas.cz>, or <tyapca7@gmail.com>
Thank You!
*/


#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

#define VERSION "0.01"

uchar maxval = 255;
uchar rmin, gmin, bmin, amin, rmax, gmax, bmax, amax;
//double  count=0;
int a, dec, hex, verbose;


// prototypes
void inspectImage(int xsize, int ysize, Memimage *m);
void usage(void);
void scanPixels(uchar **oscan, int xsize, int ysize, int nchan);
void sumPixels(uchar r, uchar g, uchar b, uchar a, uchar *rmin, uchar *gmin, uchar *bmin, uchar *amin, uchar *rmax, uchar *gmax, uchar *bmax, uchar *amax);
void print_results(void);

// functions

void
usage(void)
{
	fprint(2, "meanrgb - calculates mean values of R, G, B channels, version %s\n", VERSION);
	fprint(2, "usage: meanrgb [-?adxv] [file]\n");
	exits("usage");
}


void
sumPixels(uchar r, uchar g, uchar b, uchar a, uchar *rmin, uchar *gmin, uchar *bmin, uchar *amin, uchar *rmax, uchar *gmax, uchar *bmax, uchar *amax)
{
	if(*rmin>r) *rmin=r;
	if(*gmin>g) *gmin=g;
	if(*bmin>b) *bmin=b;
	if(*amin>a) *amin=a;

	if(*rmax<r) *rmax=r;
	if(*gmax<g) *gmax=g;
	if(*bmax<b) *bmax=b;
	if(*amax<a) *amax=a;
}


void
inspectImage(int xsize, int ysize, Memimage *m)
{
	int i, j, bpl, nchan;
	uchar **oscan;

	oscan = malloc(Dy(m->r)*sizeof(uchar*));
	if(oscan == nil)
		sysfatal("can't allocate: %r");

	/* unload original image into scan lines */
	bpl = bytesperline(m->r, m->depth);
	for(i=0; i<Dy(m->r); i++){
		oscan[i] = malloc(bpl);
		if(oscan[i] == nil)
			sysfatal("can't allocate: %r");
		j = unloadmemimage(m, Rect(m->r.min.x, m->r.min.y+i, m->r.max.x, m->r.min.y+i+1), oscan[i], bpl);
		if(j != bpl)
			sysfatal("unloadmemimage");
	}


	nchan = m->depth/8;

	// walk pixels and sum up values
	scanPixels(oscan, xsize, ysize, nchan);

}


void
scanPixels(uchar **oscan, int xsize, int ysize, int nchan)
/*
extract pixels in Y direction

oscan	image scanlines
xsize 	image dimension in X direction
ysize		image dimension in Y direction
*/
{
	int i, j;
	uchar r, g, b, a;	
//	double rsum=0, gsum=0, bsum=0, asum=0;

	for(i=0; i<xsize; i++)		// for every scanline
	{
		for(j=0; j<ysize*nchan; j+=nchan)	// jump pixels
		{
			b=oscan[i][j];
			g=oscan[i][j+1];
			r=oscan[i][j+2];
			if(nchan==4) 
			{
				a=oscan[i][j+3];
			} 
			else a=0;

			// scan for min - max values
			sumPixels( r,  g,  b,  a, &rmin, &gmin, &bmin, &amin, &rmax, &gmax, &bmax, &amax);
		}
	}
}

void 
print_results(void)
{
	if(verbose){
//		print("no. pixels= %d\n", (int)count);
		if(hex){
			if(a){
				print("min:\tR:%2.2x G:%2.2x B:%2.2x A:%2.2x\n", rmin, gmin, bmin, amin);
				print("max:\tR:%2.2x G:%2.2x B:%2.2x A:%2.2x\n", rmax, gmax, bmax, amax);
			}
			else{
				print("min:\tR:%2.2x G:%2.2x B:%2.2x\n", rmin, gmin, bmin);
				print("max:\tR:%2.2x G:%2.2x B:%2.2x\n", rmax, gmax, bmax);
			}
		}
		else{
			if(a){
				print("min:\tR:%d G:%d B:%d A:%d\n", rmin, gmin, bmin, amin);
				print("max:\tR:%d G:%d B:%d A:%d\n", rmax, gmax, bmax, amax);
			}
			else{
				print("min:\tR:%d G:%d B:%d\n", rmin, gmin, bmin);
				print("max:\tR:%d G:%d B:%d\n", rmax, gmax, bmax);
			}
		}
	}
	else if(hex){
		if(a){
			print("%2.2x %2.2x %2.2x %2.2x\n", rmin, gmin, bmin, amin);
			print("%2.2x %2.2x %2.2x %2.2x\n", rmax, gmax, bmax, amax);
		}
		else
			print("%2.2x %2.2x %2.2x\n", rmin, gmin, bmin);
			print("%2.2x %2.2x %2.2x\n", rmax, gmax, bmax);

		}
		else 	if(a){
			print("%d %d %d %d\n", rmin, gmin, bmin, amin);
			print("%d %d %d %d\n", rmax, gmax, bmax, amax);
		}
		else{
			print("%d %d %d\n", rmin, gmin, bmin);
			print("%d %d %d\n", rmax, gmax, bmax);
		}
	exits(nil);
}

void
main(int argc, char *argv[])
{
	int fd, xsize, ysize;
	Rectangle rparam;
	Memimage *m;
	char *file;
	char tmp[100];


	memimageinit();
	memset(&rparam, 0, sizeof rparam);
	xsize = ysize = 0;
	dec=1; hex=0; verbose=0; a=0;
	rmin=gmin=bmin=amin=maxval;
	rmax=gmax=bmax=amax=0;

	ARGBEGIN{
	case '?':
		usage();
		exits("usage");
		break;
	case 'a':	// alpha channel output, too
		a=1;
		break;
	case 'd':	// decimal output of R, G, B, A, in this order, default
		dec=1; hex=0;
		break;
	case 'x':	// hexadecimal output of R, G, B, A, in this order
		dec=0; hex=1;
		break;
	case 'v':	// verbose mode
		verbose=1;
		break;
	default:
		usage();
		exits("usage");
	}ARGEND

	file = "<stdin>";
	fd = 0;
	if(argc > 1)
		usage();
	else if(argc == 1){
		file = argv[0];
		fd = open(file, OREAD);
		if(fd < 0)
			sysfatal("can't open %s: %r", file);
	}

	m = readmemimage(fd);
	if(m == nil)
		sysfatal("can't read %s: %r", file);

	xsize = Dx(m->r);
	ysize = Dy(m->r);

	switch(m->chan){

	case RGB24:
	case RGBA32:
	case ARGB32:
	case XRGB32:
		inspectImage(xsize, ysize, m);		// here the computation is performed
		break;

	default:
		sysfatal("can't handle channel type %s; convert the image first", chantostr(tmp, m->chan));
	}
	print_results();
}
// EOF

