/*
swaprgb.c
Swaps old colors in R, G, B -intervals for a new RGB color
Most of this code is taken dorectly from resample.c
please report bugs and/or enhancements to:
<cej@gli.cas.cz>, or <tyapca7@gmail.com>
Thank You!
*/


#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

#define VERSION "0.01"

uchar maxval = 255;
uchar rnew, gnew, bnew, rmin, gmin, bmin, rmax, gmax, bmax;

// prototypes
Memimage* newImage(int xsize, int ysize, Memimage *m);
void usage(void);
void transformPixels(uchar **oscan, uchar **nscan, int xsize, int ysize, int nchan);
void modify_pixel(uchar *r, uchar *g, uchar *b, uchar *a);

// functions

void
usage(void)
{
	fprint(2, "swaprgb.c - swaps old colors in R, G, B -intervals for a new RGB color, version %s\n", VERSION);
	fprint(2, "usage: blah blah blah.......\n");
	exits("usage");
}


void
modify_pixel(uchar *r, uchar *g, uchar *b, uchar *a)
{
	float rr, gg, bb, aa;

	rr=*r; gg=*g; bb=*b; aa=*a; 

	if(((rr>=rmin) && (rr<=rmax)) && ((gg>=gmin) && (gg<=gmax)) && ((bb>=bmin && bb<=bmax))){
		rr=rnew; gg=gnew; bb=bnew;
	}

	// clip pixel
	if(rr>maxval) rr=maxval; 
	if(gg>maxval) gg=maxval;
	if(bb>maxval) bb=maxval;
	if(aa>maxval) aa=maxval;

	if(rr<0) rr=0;
	if(gg<0) gg=0;
	if(bb<0) bb=0;
	if(aa<0) aa=0;

	*r=floor(rr);
	*g=floor(gg);
	*b=floor(bb);
	*a=floor(aa);
}


Memimage*
newImage(int xsize, int ysize, Memimage *m)
{
	int i, j, bpl, nchan;
	Memimage *new;
	uchar **oscan, **nscan;

	new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
	if(new == nil)
		sysfatal("can't allocate new image: %r");

	oscan = malloc(Dy(m->r)*sizeof(uchar*));
	nscan = malloc(Dy(m->r)*sizeof(uchar*));
	if(oscan == nil || nscan == nil)
		sysfatal("can't allocate: %r");

	/* unload original image into scan lines */
	bpl = bytesperline(m->r, m->depth);
	for(i=0; i<Dy(m->r); i++){
		oscan[i] = malloc(bpl);
		if(oscan[i] == nil)
			sysfatal("can't allocate: %r");
		j = unloadmemimage(m, Rect(m->r.min.x, m->r.min.y+i, m->r.max.x, m->r.min.y+i+1), oscan[i], bpl);
		if(j != bpl)
			sysfatal("unloadmemimage");
	}

	/* allocate scan lines for destination. we do y first, so need at least Dy(m->r) lines */
	bpl = bytesperline(Rect(0, 0, xsize, Dy(m->r)), m->depth);
	for(i=0; i<Dy(m->r); i++){
		nscan[i] = malloc(bpl);
		if(nscan[i] == nil)
			sysfatal("can't allocate: %r");
	}

	nchan = m->depth/8;
	switch(nchan){
	case 3:
	case 4:
		break;
	default:
		sysfatal("can't handle  %d-channel images", nchan);	
	}

	transformPixels(oscan, nscan, xsize, ysize, nchan);

	/* pack data into destination */
	bpl = bytesperline(new->r, m->depth);
	for(i=0; i<ysize; i++){
		j = loadmemimage(new, Rect(0, i, xsize, i+1), nscan[i], bpl);
		if(j != bpl)
			sysfatal("loadmemimage: %r");
	}
	return new;
}


void
transformPixels(uchar **oscan, uchar **nscan, int xsize, int ysize, int nchan)
/*
extract pixels in Y direction, does something on them, and put them into new location

oscan	old scanline
nscan	new scanline
xsize 	image dimension in X direction
ysize		image dimension in Y direction
*/
{
	int i, j;
	uchar r, g, b, a;	

	for(i=0; i<xsize; i++)		// for every scanline
	{
		for(j=0; j<ysize*nchan; j+=nchan)	// jump pixels
		{
			b=oscan[i][j];
			g=oscan[i][j+1];
			r=oscan[i][j+2];
			if(nchan==4) 
			{
				a=oscan[i][j+3];
			} 
			else a=0;

			//supply the function which actually does some transformation on a pixel here:

			////////////////////////////////////////
			modify_pixel(&r, &g, &b, &a);
			////////////////////////////////////////


			// write pixel to destination scanline
			nscan[i][j]=b;
			nscan[i][j+1]=g;
			nscan[i][j+2]=r;
			if(nchan==4) nscan[i][j+3]=a;
		}
	}
}


void
main(int argc, char *argv[])
{
	int fd, xsize, ysize;
	Rectangle rparam;
	Memimage *m, *new, *t1, *t2;
	char *file;
	ulong tchan;
	char tmp[100];
//	double v;

	memimageinit();
	memset(&rparam, 0, sizeof rparam);
	xsize = ysize = 0;

	ARGBEGIN{
	case '?':
		usage();
		exits("usage");
		break;
	case 'r':
                //  red minimum
		rmin = atoi(EARGF(usage()));
		break;
	case 'R':
                //  red maximum
		rmax = atoi(EARGF(usage()));
		break;
	case 'g':
                //  green minimum
		gmin = atoi(EARGF(usage()));
		break;
	case 'G':
                //  green maximum
		gmax = atoi(EARGF(usage()));
		break;
	case 'b':
                //  blue minimum
		bmin = atoi(EARGF(usage()));
		break;
	case 'B':
                //  blue maximum
		bmax = atoi(EARGF(usage()));
		break;
	case '1':
                //  new red
		rnew = atoi(EARGF(usage()));
		break;
	case '2':
                //  new green
		gnew = atoi(EARGF(usage()));
		break;
	case '3':
                //  new blue
		bnew = atoi(EARGF(usage()));
		break;
	default:
		usage();
		exits("usage");
	}ARGEND

	file = "<stdin>";
	fd = 0;
	if(argc > 1)
		usage();
	else if(argc == 1){
		file = argv[0];
		fd = open(file, OREAD);
		if(fd < 0)
			sysfatal("can't open %s: %r", file);
	}

	m = readmemimage(fd);
	if(m == nil)
		sysfatal("can't read %s: %r", file);

	xsize = Dx(m->r);
	ysize = Dy(m->r);

	new = nil;
	switch(m->chan){

	case RGB24:
	case RGBA32:
	case ARGB32:
	case XRGB32:
		new = newImage(xsize, ysize, m);
		break;

	case GREY1:
	case GREY2:
	case GREY4:
	case GREY8:
	case CMAP8:
	case RGB15:
	case RGB16:
		tchan = RGB24;
		goto Convert;

	Convert:
		/* use library to convert to byte-per-chan form, then convert back */
		t1 = allocmemimage(m->r, tchan);
		if(t1 == nil)
			sysfatal("can't allocate temporary image: %r");
		memimagedraw(t1, t1->r, m, m->r.min, nil, ZP, S);
		t2 = newImage(xsize, ysize, t1);
		freememimage(t1);
		new = allocmemimage(Rect(0, 0, xsize, ysize), m->chan);
		if(new == nil)
			sysfatal("can't allocate new image: %r");
		/* should do error diffusion here */
		memimagedraw(new, new->r, t2, t2->r.min, nil, ZP, S);
		freememimage(t2);
		break;

	default:
		sysfatal("can't handle channel type %s", chantostr(tmp, m->chan));
	}

	assert(new);
	if(writememimage(1, new) < 0)
		sysfatal("write error on output: %r");

	exits(nil);
}
// EOF

